@@ -7,7 +7,10 @@
 """
 import evolution
 import gtk
-	
+
+import logging
+log = logging.getLogger('fmms.%s' % __name__)
+
 class ContactHandler:
 	
 	
@@ -40,13 +43,22 @@ class ContactHandler:
 			if nr != None:
 				nrlist.append(nr)"""
 		# creative use of processing power? *cough*
-		nrlist = []
+		nrlist = {}
 		vcardlist = res.get_vcard_string().replace('\r', '').split('\n')
 		for line in vcardlist:
 			if line.startswith("TEL"):
 				nr = line.split(":")[1]
+				ltype = line.split(":")[0].split("=")
+				phonetype = "Unknown"
+				try:
+					for type in ltype:
+						rtype = type.replace(";TYPE", "")
+						if rtype != "TEL" and rtype != "VOICE":
+							phonetype = rtype	
+				except:
+					pass
 				if nr != None:
-					nrlist.append(nr)
+					nrlist[nr] = phonetype
 		return nrlist
 		
 	
@@ -77,12 +89,13 @@ class ContactHandler:
 					
 		return None
 	
+	# TODO: get from uid instead of name
 	def get_photo_from_name(self, pname, imgsize):
 		res = self.ab.search(pname)
 		### do some nice stuff here
-		#l = [x.get_name() for x in res]
-		#print "search for:", pname, "gave res: ", l
-		if res != None:
+		l = [x.get_name() for x in res]
+		log.info("search for: %s gave: %s (%s)", pname, l, l.__class__)
+		if res != None and res.__class__ == list:
 			img = res[0].get_photo(imgsize)
 			if img == None:
 				vcardlist = res[0].get_vcard_string().replace('\r', '').split('\n') # vcard for first result
@@ -94,8 +107,6 @@ class ContactHandler:
 						if height != imgsize:
 							newheight = imgsize
 							newwidth = int(newheight * img.get_width() / height)
-							#print "h:", height, "w:", img.get_width()
-							#print "newh:", newheight, "neww:", newwidth
 							img = img.scale_simple(newwidth, newheight, gtk.gdk.INTERP_BILINEAR)
 			return img
 		else:
@@ -104,22 +115,4 @@ class ContactHandler:
 		
 if __name__ == '__main__':
 	cb = ContactHandler()
-	phonedict = {}
-	print cb.get_name_from_number("0730354431")
-	#print cb.ab.get_all_contacts()[0].__doc__
-	#asd = cb.get_contacts_as_list()
-	#print asd
-	
-	"""r = cb.ab.search('Tom Le') # Returns List of results
-	print r
-	l = [x.get_name() for x in r] # list of results
-	u = r[0].get_name() # name of the first result
-	vcardlist = r[0].get_vcard_string().replace('\r', '').split('\n') # vcard for first result
-	#print vcardlist"""
-	#print cb.get_numbers_from_name('')
-	"""
-	for line in vcardlist:
-		if not line.startswith("PHOTO"):
-			print line
-	"""
-	#print r[0].get_photo(64)
+	#c = ab.get_contact("id")
