@@ -1,29 +1,34 @@
 #!/usr/bin/env python2.5
 # -*- coding: utf-8 -*-
-""" UI for fMMS
+""" Main-view UI for fMMS
 
 @author: Nick Leppänen Larsson <frals@frals.se>
 @license: GNU GPL
 """
-import gtk
-gtk.gdk.threads_init()
-import hildon
 import os
 import time
-from wappushhandler import PushHandler
-import locale
+
+import gtk
+import hildon
 import osso
-import fmms_config as fMMSconf
-import fmms_sender_ui as fMMSSenderUI
-import contacts as ContactH
 import gobject
 import dbus
-import mimetypes
 from gnome import gnomevfs
 
+from wappushhandler import PushHandler
+import fmms_config as fMMSconf
+import fmms_sender_ui as fMMSSenderUI
+import fmms_viewer as fMMSViewer
+import controller as fMMSController
+import contacts as ContactH
+
+import logging
+log = logging.getLogger('fmms.%s' % __name__)
+
 class fMMS_GUI(hildon.Program):
 
 	def __init__(self):
+		self.cont = fMMSController.fMMS_controller()
 		self.config = fMMSconf.fMMS_config()
 		self._mmsdir = self.config.get_mmsdir()
 		self._pushdir = self.config.get_pushdir()
@@ -31,10 +36,10 @@ class fMMS_GUI(hildon.Program):
 		self.osso_c = osso.Context("fMMS", "0.1.0", False)
 	
 		if not os.path.isdir(self._mmsdir):
-			print "creating dir", self._mmsdir
+			log.info("creating dir %s", self._mmsdir)
 			os.makedirs(self._mmsdir)
 		if not os.path.isdir(self._pushdir):
-			print "creating dir", self._pushdir
+			log.info("creating dir %s", self._pushdir)
 			os.makedirs(self._pushdir)
 	
 		hildon.Program.__init__(self)
@@ -98,36 +103,47 @@ class fMMS_GUI(hildon.Program):
 		self.treeview.tap_and_hold_setup(self.liststore_menu)
 		#treeview.connect('tap-and-hold', self.liststore_mms_clicked)
 		
-	
+		
 		pan.add_with_viewport(self.treeview)
 		self.window.add(pan)
 	
 		self.menu = self.create_menu()
 		self.window.set_app_menu(self.menu)
+		
+		self.window.connect('focus-in-event', self.cb_on_focus)
+		
 		self.window.show_all()
 		self.add_window(self.window)
 		
 		if self.config.get_firstlaunch() == 1:
-			print "firstlaunch"
 			note = osso.SystemNote(self.osso_c)
 			firstlaunchmessage = "NOTE: Currently you have to connect manually to the MMS APN when sending and receiving.\nAlso, only implemented attachment is image."
 			note.system_note_dialog(firstlaunchmessage , 'notice')
 			self.create_config_dialog()
 			self.config.set_firstlaunch(0)
-		
+	
+	def cb_on_focus(self, widget, event):
+		# TODO: re-set iter
+		#log.info("got focus")
+		#self.liststore.clear()
+		#self.add_buttons_liststore()
+		pass
+	
 	def cb_open_fmms(self, interface, method, args, user_data):
 		if method != 'open_mms' and method != 'open_gui':
 			return
 		if method == 'open_mms':
-			try:
-				checkfile = os.path.isfile(self._pushdir + args[0])
-				if checkfile == True:
-					filename = args[0]
-			except:
+			filename = args[0]
+			if self.cont.is_fetched_push_by_transid(filename):
+				hildon.hildon_gtk_window_set_progress_indicator(self.window, 1)
+				banner = hildon.hildon_banner_show_information(self.window, "", "fMMS: Opening message")
+				self.force_ui_update()
+				viewer = fMMSViewer.fMMS_Viewer(filename)
+				hildon.hildon_gtk_window_set_progress_indicator(self.window, 0)
+				return
+			else:
 				return
-			self.show_new_window(filename)
 		elif method == 'open_gui':
-			print "open_gui called"
 			self.liststore.clear()
 			self.add_buttons_liststore()
 			return
@@ -247,14 +263,12 @@ class fMMS_GUI(hildon.Program):
 		dialog.destroy()
 		return ret
 
+
 	""" selector for apn """
 	def create_apn_selector(self):
 		selector = hildon.TouchSelector(text = True)
-		
 		apnlist = self.config.get_gprs_apns()
-		
 		currval = self.config.get_apn_nicename()
-		
 		# Populate selector
 		i = 0
 		for apn in apnlist:
@@ -274,50 +288,57 @@ class fMMS_GUI(hildon.Program):
 		
 	def config_menu_button_clicked(self, action):
 		if action == gtk.RESPONSE_APPLY:
-			print self.apn.get_selector().get_current_text()
+			log.info("%s", self.apn.get_selector().get_current_text())
 			ret_setapn = self.config.get_apnid_from_name(self.apn.get_selector().get_current_text())
 			if ret_setapn != None:
 				self.config.set_apn(ret_setapn)
-				print "Set apn to: %s" % ret_setapn
+				log.info("Set apn to: %s" % ret_setapn)
 				ret = self.config.set_mmsc(self.mmsc.get_text())
-				print "Set mmsc to %s" % self.mmsc.get_text()
+				log.info("Set mmsc to %s" % self.mmsc.get_text())
 				self.config.set_phonenumber(self.number.get_text())
-				print "Set phonenumber to %s" % self.number.get_text()
+				log.info("Set phonenumber to %s" % self.number.get_text())
 				self.config.set_img_resize_width(self.imgwidth.get_text())
-				print "Set image width to %s" % self.imgwidth.get_text()
+				log.info("Set image width to %s" % self.imgwidth.get_text())
 				banner = hildon.hildon_banner_show_information(self.window, "", "Settings saved")
 				return 0
 			else:
-				print "Set mmsc to %s" % self.mmsc.get_text()
+				log.info("Set mmsc to %s" % self.mmsc.get_text())
 				self.config.set_phonenumber(self.number.get_text())
-				print "Set phonenumber to %s" % self.number.get_text()
+				log.info("Set phonenumber to %s" % self.number.get_text())
 				self.config.set_img_resize_width(self.imgwidth.get_text())
-				print "Set image width to %s" % self.imgwidth.get_text()
+				log.info("Set image width to %s" % self.imgwidth.get_text())
 				banner = hildon.hildon_banner_show_information(self.window, "", "Could not save APN settings. Did you enter a correct APN?")
 				banner.set_timeout(5000)
 				return -1
 		
-	""" sorts files based on mtime, newest first """
-	def getfiles(self, dirpath):
-		a = [s for s in os.listdir(dirpath)]
-		a.sort(key=lambda s: os.path.getmtime(os.path.join(dirpath, s)))
-		a.reverse()
-		return a
 
 	""" add each item to our liststore """
 	def add_buttons_liststore(self):
 			icon_theme = gtk.icon_theme_get_default()
 			
-			filelist = self.getfiles(self._pushdir)
-			for fname in filelist:
-				mtime = time.strftime("%Y-%m-%d | %H:%M", time.localtime(os.path.getmtime(self._pushdir + fname)))
-				varlist = self._read_push_as_list(self._pushdir + fname)
+			pushlist = self.cont.get_push_list()
+			for varlist in pushlist:
+				mtime = varlist['Time']
+				fname = varlist['Transaction-Id']
+				direction = self.cont.get_direction_mms(fname)
+				# TODO Use fancy icon for showing read/unread
+				
+				
+				if self.cont.is_mms_read(fname):
+					isread = " (Read)"
+				else:
+					isread = " (Unread)"
+				
+				
 				try:
 					sender = varlist['From']
 					sender = sender.replace("/TYPE=PLMN", "")
 				except:
 					sender = "0000000"
 				
+				if direction == fMMSController.MSG_DIRECTION_OUT:
+					sender = "You (Outgoing)"
+				
 				sendername = self.ch.get_name_from_number(sender)
 				photo = icon_theme.load_icon("general_default_avatar", 48, 0)
 				if sendername != None:
@@ -325,67 +346,47 @@ class fMMS_GUI(hildon.Program):
 					phototest = self.ch.get_photo_from_name(sendername, 64)
 					if phototest != None:	
 						photo = phototest
-						#print "loaded photo:", photo.get_width(), photo.get_height()
+						#log.info("loaded photo: %s %s", photo.get_width(), photo.get_height())
 	
 				#title = sender + " - " + mtime
 				
-				if self._is_fetched_push(fname):
+				if self.cont.is_fetched_push_by_transid(fname):
 					icon = icon_theme.load_icon("general_sms", 48, 0)
 				else:
 					icon = icon_theme.load_icon("chat_unread_sms", 48, 0)
-				self.liststore.append([icon, sender + '  <span foreground="#666666" size="smaller"><sup>' + mtime + '</sup></span>\n<span foreground="#666666" size="x-small">' + fname + '</span>', photo, fname])
+				self.liststore.append([icon, sender + isread + '  <span foreground="#666666" size="smaller"><sup>' + mtime + '</sup></span>\n<span foreground="#666666" size="x-small">' + fname + '</span>', photo, fname])
+	
 	""" lets call it quits! """
 	def quit(self, *args):
 		gtk.main_quit()
 	
+	
 	""" forces ui update, kinda... god this is AWESOME """
 	def force_ui_update(self):
 		while gtk.events_pending():
 			gtk.main_iteration(False)
 		
+		
 	""" delete push message """
 	def delete_push(self, fname):
-		fullpath = self._pushdir + fname
-		print fullpath
-		if os.path.isfile(fullpath):
-			print "removing", fullpath
-			try:
-				os.remove(fullpath)
-			except Exception, e:
-				raise
+		self.cont.delete_push_message(fname)
 		
 	
 	""" delete mms message (eg for redownload) """
 	def delete_mms(self, fname):
-		fullpath = self._mmsdir + fname
-		print fullpath
-		if os.path.isdir(fullpath):
-			print "starting deletion of", fullpath
-			filelist = self.getfiles(fullpath)
-			for fn in filelist:
-				try:
-					fullfn = fullpath + "/" + fn
-					os.remove(fullfn)
-				except:
-					print "failed to remove", fullfn
-			try:
-				print "trying to remove", fullpath
-				os.rmdir(fullpath)
-			except OSError, e:
-				print "failed to remove dir:", e, e.args
-				raise
+		self.cont.delete_mms_message(fname)
 	
 	""" delete push & mms """
 	def delete_push_mms(self, fname):
 		try:
-			self.delete_push(fname)
-			self.delete_mms(fname)
+			self.cont.wipe_message(fname)
 			banner = hildon.hildon_banner_show_information(self.window, "", "fMMS: Message deleted")
 		except Exception, e:
-			print "Exception caught:"
-			print e, e.args
+			log.exception("%s %s", type(e), e)
+			raise
 			banner = hildon.hildon_banner_show_information(self.window, "", "fMMS: Failed to delete message.")
 
+
 	""" action on delete contextmenu click """
 	def liststore_delete_clicked(self, widget):
 		dialog = gtk.Dialog()
@@ -400,14 +401,13 @@ class fMMS_GUI(hildon.Program):
 			(model, miter) = self.treeview.get_selection().get_selected()
 			# the 4th value is the filename (start counting at 0)
 			filename = model.get_value(miter, 3)
-			print "deleting", filename
+			log.info("deleting %s", filename)
 			self.delete_push_mms(filename)
 			self.liststore.remove(miter)
 		dialog.destroy()
 		return
 	
 	""" action on redl contextmenu click """
-	# TODO: dont block on this?
 	def liststore_redl_clicked(self, widget):
 		hildon.hildon_gtk_window_set_progress_indicator(self.window, 1)
 		dialog = gtk.Dialog()
@@ -425,13 +425,19 @@ class fMMS_GUI(hildon.Program):
 			(model, miter) = self.treeview.get_selection().get_selected()
 			# the 4th value is the filename (start counting at 0)
 			filename = model.get_value(miter, 3)
-			print "redownloading", filename
+			log.info("redownloading %s", filename)
 			try:
 				self.delete_mms(filename)
-				self._get_mms_from_push(self._pushdir + "/" + filename)
+				banner = hildon.hildon_banner_show_information(self.window, "", "fMMS: Trying to download MMS...")
+				self.force_ui_update()
+				
+				# TODO: FIXME
+				
+				self.cont.get_mms_from_push(filename)
 				self.show_mms(self.treeview, model.get_path(miter))
 			except Exception, e:
-				print e, e.args
+				log.exception("%s %s", type(e), e)
+				#raise
 				banner = hildon.hildon_banner_show_information(self.window, "", "fMMS: Operation failed")
 			hildon.hildon_gtk_window_set_progress_indicator(self.window, 0)
 		return
@@ -465,251 +471,25 @@ class fMMS_GUI(hildon.Program):
 		banner = hildon.hildon_banner_show_information(self.window, "", "fMMS: Opening message")
 		self.force_ui_update()
 		
-		print path
+		log.info("showing mms: %s", path)
 		model = treeview.get_model()
 		miter = model.get_iter(path)
-		# the 4th value is the filename (start counting at 0)
-		filename = model.get_value(miter, 3)
+		# the 4th value is the transactionid (start counting at 0)
+		transactionid = model.get_value(miter, 3)
 		
 		try:
-			self.show_new_window(filename)
-		except:
-			pass
-		# this is ugly i know, but i really want it to work!
+			viewer = fMMSViewer.fMMS_Viewer(transactionid)
+		except Exception, e:
+			log.exception("%s %s", type(e), e)
+			#raise
+		
+		
 		hildon.hildon_gtk_window_set_progress_indicator(self.window, 0)
-		
-	""" open a new window containing mms specified by fname """		
-	def show_new_window(self, fname):
-		win = hildon.StackableWindow()
-		win.set_title("Showing MMS: " + fname)
-	
-		vbox = gtk.VBox()
-		pan = hildon.PannableArea()
-		pan.set_property("mov-mode", hildon.MOVEMENT_MODE_BOTH)
-		
-		self._parse_mms(fname, vbox)
-		
-		pan.add_with_viewport(vbox)
-		win.add(pan)
-		
-		mms_menu = self.create_mms_menu(fname)
-		win.set_app_menu(mms_menu)
-		win.show_all()
-
-	""" create app menu for mms viewing window """
-	def create_mms_menu(self, fname):
-		menu = hildon.AppMenu()
-		
-		headers = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
-		headers.set_label("Headers")
-		headers.connect('clicked', self.mms_menu_button_clicked, fname)
-		
-		menu.append(headers)
-	
-		menu.show_all()
-		
-		return menu		
-	
-	""" actions for mms menu """
-	def mms_menu_button_clicked(self, button, fname):
-		buttontext = button.get_label()
-		if buttontext == "Headers":
-			ret = self.create_headers_dialog(fname)
-
-	""" show headers in a dialog """
-	def create_headers_dialog(self, fname):
-		dialog = gtk.Dialog()
-		dialog.set_title("Headers")
-		
-		dialogVBox = gtk.VBox()
-		
-		pan = hildon.PannableArea()
-		pan.set_property("mov-mode", hildon.MOVEMENT_MODE_BOTH)
-		pan.set_property("size-request-policy", hildon.SIZE_REQUEST_CHILDREN)
-		
-		allVBox = gtk.VBox()
-		
-		fullpath = self._mmsdir + "/" + str(fname) + "/headers"
-		try: 
-			fp = open(fullpath, 'r')
-			for line in fp:
-				line = line.replace("\n", "")
-				label = gtk.Label(line)
-				label.set_alignment(0, 0)
-				#pan.add(label)
-				allVBox.pack_start(label, False, False, 0)
-		except:
-			pass
-			
-		allVBox.show_all()
-		
-		pan.add_with_viewport(allVBox)
-		dialog.vbox.add(pan)
-		dialog.vbox.show_all()
-		ret = dialog.run()
-		
-		dialog.destroy()
-		return ret
-
-	""" this function takes the FILENAME, not the full path """
-	def _is_fetched_push(self, filename):
-		path = self._mmsdir + filename
-		if os.path.isdir(path):
-			if os.path.isfile(self._mmsdir + filename + "/message"):
-				return True
-		else:
-			return False
-	
-	""" parse mms and push each part to the container 
-	    fetches the mms if its not downloaded         """
-	# TODO: better file detection/support
-	def _parse_mms(self, filename, container):
-		hildon.hildon_gtk_window_set_progress_indicator(self.window, 1)
-		self.force_ui_update()
-		
-		if not self._is_fetched_push(filename):
-			self._get_mms_from_push(self._pushdir + filename)
-				
-		textview = gtk.TextView()
-		textview.set_editable(False)
-		textview.set_cursor_visible(False)
-		textview.set_wrap_mode(gtk.WRAP_WORD)
-		textbuffer = gtk.TextBuffer()
-		path = self._mmsdir + filename
-		filelist = os.listdir(path)
-		for fname in filelist:
-			(name, ext) = os.path.splitext(fname)
-			fnpath = os.path.join(path, fname)
-			isText = False
-			isImage = False
-			try:
-				filetype = mimetypes.guess_type(fnpath)[0]
-				print "filetype:", filetype
-				if filetype != None:
-					if filetype.startswith("image"):
-						isImage = True
-					if filetype.startswith("text"):
-						isText = True
-			except Exception, e:
-				filetype = None
-				print e, e.args
-			
-			# TODO: guess on content-type not fileextension, really
-			if isImage or ext == ".wbmp":
-				""" insert the image in an eventbox so we can get signals """
-				ebox = gtk.EventBox()
-				img = gtk.Image()
-				img.set_from_file(path + "/" + fname)
-				fullpath = path + "/" + fname
-				ebox.add(img)
-				## TODO: make this menu proper without this ugly
-				# args passing
-				menu = self.mms_img_menu(fullpath)
-				ebox.tap_and_hold_setup(menu)
-				#ebox.connect('tap-and-hold', self.mms_img_clicked)
-				container.add(ebox)
-			elif isText or ext.startswith(".txt"):
-				fp = open(path + "/" + fname, 'r')
-				contents = fp.read()
-				fp.close()
-				#print contents
-				textbuffer.insert(textbuffer.get_end_iter(), contents)
-			elif name != "message" and name != "headers" and not ext.startswith(".smil"):
-				attachButton = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL, fname)
-				attachButton.connect('clicked', self.mms_img_clicked, fnpath)
-				container.pack_end(attachButton, False, False, 0)
-				
-		textview.set_buffer(textbuffer)
-		container.add(textview)
-		hildon.hildon_gtk_window_set_progress_indicator(self.window, 0)
-		
-		
-	""" action on click on image/button """
-	def mms_img_clicked(self, widget, data):
-		print widget, data
-		path = "file://" + data
-		# gnomevfs seems to be better than mimetype when guessing mimetype for us
-		file_mimetype = gnomevfs.get_mime_type(path)
-		if file_mimetype != None:
-			if file_mimetype.startswith("video") or file_mimetype.startswith("audio"):
-				rpc = osso.Rpc(self.osso_c)
-				rpc.rpc_run("com.nokia.mediaplayer", "/com/nokia/mediaplayer", "com.nokia.mediaplayer", "mime_open", (str, path))	
-			elif file_mimetype.startswith("image"):
-				rpc = osso.Rpc(self.osso_c)
-				rpc.rpc_run("com.nokia.image_viewer", "/com/nokia/image_viewer", "com.nokia.image_viewer", "mime_open", (str, path))
-		else:
-			# TODO: how to solve this?
-			# move .mms to ~/MyDocs? change button to copy file to ~/MyDocs?
-			#rpc = osso.Rpc(self.osso_c)
-			#path = os.path.dirname(path).replace("file://", "")
-			print path
-			#rpc.rpc_run("com.nokia.osso_filemanager", "/com/nokia/osso_filemanager", "com.nokia.osso_filemanager", "open_folder", (str, path))
-
-
-	""" long press on image creates this """
-	def mms_img_menu(self, data=None):
-		print "menu created"
-		menu = gtk.Menu()
-		menu.set_title("hildon-context-sensitive-menu")
-
-		openItem = gtk.MenuItem("Open")
-		menu.append(openItem)
-		openItem.connect("activate", self.mms_img_clicked, data)
-		openItem.show()
-		menu.show_all()
-		return menu
-
-	
-	
-	""" reads a saved push message into a dict """
-	def _read_push_as_list(self, fname):
-		fp = open(fname, 'r')
-		pdict = {}
-		for line in fp:
-			line = line.replace("\n", "")
-			lsplit = line.partition(" ")
-			pdict[lsplit[0]] = lsplit[2]
-		fp.close()
-		#print pdict
-		return pdict
-		
-	""" gets a mms from a previously received push """
-	""" this function requires the fname to be the fullpath """
-	# TODO: dont require fullpath
-	def _get_mms_from_push(self, fname):
-		
-		plist = self._read_push_as_list(fname)
-		try:
-			sndr = plist['From']
-		except:
-			sndr = "0000000"
-		url = plist['Content-Location']
-		print url
-		trans_id = plist['Transaction-Id']
-		print trans_id
-		
-		PUSH = PushHandler()
-		
-		### ugly hack as well
-		# TODO: clean up
-		hildon.hildon_banner_show_information(self.window, "", "fMMS: Attempting to download MMS from server.")
-		self.force_ui_update()
-		
-		path = PUSH._get_mms_message(url, trans_id)
-		PUSH.decodeMMS(path)
-		
-		#PUSH._create_html(path, sndr)
-		return 0
-
 
 	def run(self):
 		self.window.show_all()
-		#gtk.gdk.threads_enter()
 		gtk.main()
-		#gtk.gdk.threads_leave()
 		
 if __name__ == "__main__":
 	app = fMMS_GUI()
-	app.run()
-	
-	
\ No newline at end of file
+	app.run()
\ No newline at end of file
