@@ -1,24 +1,31 @@
 #!/usr/bin/env python2.5
 # -*- coding: utf-8 -*-
-""" UI for fMMS
+""" Sender UI for fMMS
 
 @author: Nick Leppänen Larsson <frals@frals.se>
 @license: GNU GPL
 """
-import gtk
-import hildon
 import os
 import time
-import gobject
 import socket
-from wappushhandler import MMSSender
-import osso
-import fmms_config as fMMSconf
-import contacts as ContactH
+import re
 import Image
 import mimetypes
+
+import gtk
+import hildon
+import gobject
+import osso
 import dbus
 
+from wappushhandler import MMSSender
+import fmms_config as fMMSconf
+import contacts as ContactH
+import controller as fMMSController
+
+import logging
+log = logging.getLogger('fmms.%s' % __name__)
+
 class fMMS_GUI(hildon.Program):
 	def __init__(self, spawner=None):
 		hildon.Program.__init__(self)
@@ -26,6 +33,7 @@ class fMMS_GUI(hildon.Program):
 		
 		self.config = fMMSconf.fMMS_config()
 		self.ch = ContactH.ContactHandler()
+		self.cont = fMMSController.fMMS_controller()
 		
 		self.window = hildon.StackableWindow()
 		self.window.set_title("fMMS - New MMS")
@@ -46,8 +54,6 @@ class fMMS_GUI(hildon.Program):
 		bTo.connect('clicked', self.open_contacts_dialog)
 		self.eNumber = hildon.Entry(gtk.HILDON_SIZE_FINGER_HEIGHT)
 		
-		#topHBox.add(bTo)
-		#topHBox.add(eNumber)
 		topHBox1.pack_start(bTo, False, True, 0)
 		topHBox1.pack_start(self.eNumber, True, True, 0)
 		
@@ -56,11 +62,9 @@ class fMMS_GUI(hildon.Program):
 		pan = hildon.PannableArea()
 		pan.set_property("mov-mode", hildon.MOVEMENT_MODE_BOTH)		
 		
-		#midHBox = gtk.HBox()
 		self.tvMessage = hildon.TextView()
 		self.tvMessage.set_wrap_mode(gtk.WRAP_WORD)
 		
-		#midHBox.pack_start(self.tvMessage, True, True, 0)
 		pan.add_with_viewport(self.tvMessage)
 		
 		""" Begin botsection """
@@ -74,7 +78,7 @@ class fMMS_GUI(hildon.Program):
 		self.lSize = gtk.Label('')
 		
 		self.bSend = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL, "    Send    ")
-		self.bSend.connect('clicked', self.send_mms)
+		self.bSend.connect('clicked', self.send_mms_clicked)
 		
 		botHBox.pack_start(self.bAttachment)
 		botHBox.pack_start(self.lSize)
@@ -83,14 +87,9 @@ class fMMS_GUI(hildon.Program):
 
 		""" Show it all! """
 		allBox.pack_start(topHBox1, False, False)
-		#allBox.pack_start(topHBox2, False, False)
-		#allBox.pack_start(midHBox, True, True)
 		allBox.pack_start(pan, True, True)
 		allBox.pack_start(botHBox, False, False)
 		
-		#self.pan = pan
-		#self.pan.add_with_viewport(allBox)
-		#self.window.add(self.pan)
 		self.window.add(allBox)
 		self.window.show_all()
 		self.add_window(self.window)
@@ -124,6 +123,7 @@ class fMMS_GUI(hildon.Program):
 	def contact_number_chosen(self, button, nrdialog):
 		print button.get_label()
 		nr = button.get_label().replace(" ", "")
+		nr = re.sub("[^0-9]\+", "", nr)
 		self.eNumber.set_text(nr)
 		nrdialog.response(0)
 		self.contacts_dialog.response(0)
@@ -131,17 +131,20 @@ class fMMS_GUI(hildon.Program):
 	def contact_selector_changed(self, selector):
 		username = selector.get_current_text()
 		nrlist = self.ch.get_numbers_from_name(username)
-		print nrlist
 		nrdialog = gtk.Dialog("Pick a number")
 		for number in nrlist:
+			numberbox = gtk.HBox()
+			typelabel = gtk.Label(nrlist[number].capitalize())
+			typelabel.set_width_chars(24)
 			button = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
 			button.set_label(number)
 			button.connect('clicked', self.contact_number_chosen, nrdialog)
-			nrdialog.vbox.pack_start(button)
+			numberbox.pack_start(typelabel, False, False, 0)
+			numberbox.pack_start(button, True, True, 0)
+			nrdialog.vbox.pack_start(numberbox)
 		nrdialog.show_all()
 		# this is blocking until we get a return
 		ret = nrdialog.run()
-		print "changed ret:", ret
 		nrdialog.destroy()
 		return ret
 	
@@ -192,21 +195,20 @@ class fMMS_GUI(hildon.Program):
 	def resize_img(self, filename):
 		try:
 			if not os.path.isdir(self.config.get_imgdir()):
-				print "creating dir", self.config.get_imgdir()
+				log.info("creating dir %s", self.config.get_imgdir())
 				os.makedirs(self.config.get_imgdir())
 			
 			hildon.hildon_banner_show_information(self.window, "", "fMMS: Resizing image, this might take a while...")
 			self.force_ui_update()
 			
 			img = Image.open(filename)
-			print "height", img.size[1]
-			print "width", img.size[0]
+			log.info("width %s", str(img.size[0]))
+			log.info("height %s", str(img.size[1]))
 			newWidth = int(self.config.get_img_resize_width())
 			if img.size[0] > newWidth:
-				print "resizing"
 				newWidth = int(self.config.get_img_resize_width())
 				newHeight = int(newWidth * img.size[1] / img.size[0])
-				print "Resizing image:", str(newWidth), "*", str(newHeight)
+				log.info("Resizing image: %s * %s", str(newWidth), str(newHeight))
 
 				# Image.BILINEAR, Image.BICUBIC, Image.ANTIALIASING
 				rimg = img.resize((newWidth, newHeight), Image.BILINEAR)
@@ -216,39 +218,48 @@ class fMMS_GUI(hildon.Program):
 				rimg.save(rattachment)
 				self.attachmentIsResized = True
 			else:
-				print "not resizing"
 				rattachment = filename
 				
 			return rattachment
 		
 		except Exception, e:
-			print "resizing failed:", e, e.args
+			log.exception("resizer: %s %s", type(exc), exc)
 			raise
 	
+	def send_mms_clicked(self, widget):
+		# Disable send-button
+		self.bSend.set_sensitive(False)
+		self.send_mms(widget)
+		hildon.hildon_gtk_window_set_progress_indicator(self.window, 0)
+		self.bSend.set_sensitive(True)
+	
 	""" sends the message (no shit?) """
 	def send_mms(self, widget):
 		hildon.hildon_gtk_window_set_progress_indicator(self.window, 1)
-		# Disable send-button
-		self.bSend.set_sensitive(False)
 		self.force_ui_update()
 		
 		self.osso_c = osso.Context("fMMS", "0.1.0", False)
 		
+		
+		to = self.eNumber.get_text()
+		if not self.cont.validate_phonenumber(to):
+			note = osso.SystemNote(self.osso_c)
+			note.system_note_dialog("Invalid phonenumber, must only contain + and digits" , 'notice')
+			return
+		
 		attachment = self.bAttachment.get_label()
 		if attachment == "Attachment" or attachment == None:
 			attachment = None
 			self.attachmentIsResized = False
 		else:
-			print attachment
+			log.info("attachment: %s", attachment)
 			filetype = mimetypes.guess_type(attachment)[0]
-			print self.config.get_img_resize_width()
 			self.attachmentIsResized = False
-			print filetype.startswith("image")
 			if self.config.get_img_resize_width() != 0 and filetype.startswith("image"):
 				try:
 					attachment = self.resize_img(attachment)
 				except Exception, e:
-					print e, e.args
+					log.exception("resize failed: %s %s", type(exc), exc)
 					note = osso.SystemNote(self.osso_c)
 					errmsg = str(e.args)
 					note.system_note_dialog("Resizing failed:\nError: " + errmsg , 'notice')
@@ -258,10 +269,11 @@ class fMMS_GUI(hildon.Program):
 		sender = self.config.get_phonenumber()
 		tb = self.tvMessage.get_buffer()
 		message = tb.get_text(tb.get_start_iter(), tb.get_end_iter())
-		print sender, attachment, to, message
+		log.info("sender: %s attachment %s to %s message %s", sender, attachment, to, message)
 
 		""" Construct and send the message, off you go! """
 		# TODO: remove hardcoded subject
+		# TODO: let controller do this
 		try:
 			sender = MMSSender(to, "MMS", message, attachment, sender)
 			(status, reason, output) = sender.sendMMS()
@@ -275,14 +287,13 @@ class fMMS_GUI(hildon.Program):
 			banner = hildon.hildon_banner_show_information(self.window, "", "MMSC REPLIED:" + message + "\nBODY: " + reply)
                         
 		except TypeError, exc:
-			#print type(exc)
+			log.exception("sender: %s %s", type(exc), exc)
 			note = osso.SystemNote(self.osso_c)
 			errmsg = "Invalid attachment"
 			note.system_note_dialog("Sending failed:\nError: " + errmsg + " \nPlease make sure the file is valid" , 'notice')
 			#raise
 		except socket.error, exc:
-			#print type(exc)
-			print exc
+			log.exception("sender: %s %s", type(exc), exc)
 			code = str(exc.args[0])
 			text = str(exc.args[1])
 			note = osso.SystemNote(self.osso_c)
@@ -290,15 +301,14 @@ class fMMS_GUI(hildon.Program):
 			note.system_note_dialog("Sending failed:\nError: " + errmsg + " \nPlease make sure APN settings are correct" , 'notice')
 			#raise
 		except Exception, exc:
-			print type(exc)
-			print exc
-			#raise
+			log.exception("sender: %s %s", type(exc), exc)
+			raise
 		finally:
 			hildon.hildon_gtk_window_set_progress_indicator(self.window, 0)
 			self.bSend.set_sensitive(True)
 			
 		if self.attachmentIsResized == True:
-			print "Removing temporary image..."
+			log.info("Removing temporary image: %s", attachment)
 			os.remove(attachment)
 		#self.window.destroy()
 		
